<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class Admin_UserController extends Controller
{ 
    protected $models = array("user", "bannedIp", "task");
    
    /** @var UserModel*/ protected $user;
    /** @var BannedIpModel*/ protected $bannedIp; 
    /** @var TaskModel*/ protected $task; 
    
    function init()
    {        
        $this->acl->allow('administrator', $this->name, "*");
         
        if(!$this->acl->isAllowed($this->session->get("role"), $this->name, $this->action))
        {
            $this->redirect(FrontController::getRewrittedUrl("/admin/main/logIn"));  
        }
    }   
    
    function indexAction()
    {                    

    }
    
    function saveNewAdminPasswordAction()
    {
        if(!empty($this->request->newPassword)
        && $this->request->newPassword == $this->request->repeatedNewPassword)
        {
            $c = new Criteria();
            $c->add("role", "administrator");
            $this->user->update(array("password" => md5($this->request->newPassword)), $c);
        }
        
        $this->redirect($this->moduleLink());
    }
    
    function moderatorAction()
    {
        $moderators = $this->user->getUsersWithRole("moderator");
        $stats = $this->statistic->getModeratorsStats();
        
        
        foreach($moderators as $moderator)
        {
            $moderator->validatedCount = isset($stats['validated'][$moderator->userId]['validatedSitesCount']) ? $stats['validated'][$moderator->userId]['validatedSitesCount'] : 0;;
            $moderator->refusedCount = isset($stats['refused'][$moderator->userId]['refusedSitesCount']) ? $stats['refused'][$moderator->userId]['refusedSitesCount'] : 0;
            $moderator->bannedCount = isset($stats['banned'][$moderator->userId]['bannedSitesCount']) ? $stats['banned'][$moderator->userId]['bannedSitesCount'] : 0;
        }
        
        $this->set("moderators", $moderators);
    }
    
    function updateModeratorAction($moderatorId)
    {
        if(isset($this->request->delete))
        {
            $this->deleteModeratorAction($moderatorId);
        }
        else
        {
            $this->user->updateByPk(array("email" => $this->request->email), $moderatorId);
            $this->redirect($this->moduleLink("moderator"));
        }
    }
    
    function deleteModeratorAction($moderatorId)
    {
        $this->user->delByPk($moderatorId);
        $this->redirect($this->moduleLink("moderator"));
    }
    
    function saveNewModeratorAction()
    {
        if(!empty($this->request->login) && !empty($this->request->password))
        {
            $moderator = new UserRecord();
            $moderator->email = $this->request->login;
            $moderator->password = md5($this->request->password);
            $moderator->role = "moderator";
            $moderator->save();
            
            $refusal = new RefusalRecord();
            $refusal->moderatorId = $moderator->userId;
            $refusal->refusedSitesCount = 0;
            $refusal->save();
        }
        
        $this->redirect($this->moduleLink("moderator")); 
    }
    
    function webmasterAction($page = 1)
    {
        $c = new Criteria();
        $c->add("role", "webmaster");
        
        //set pagination
        $perPage = 50;
        $totalCount = $this->user->getCount($c);
        $totalPages = ceil($totalCount / $perPage);

        $this->set("pageNavigation", array("baseLink" => "/admin/user/webmaster/",
                                           "totalPages" => $totalPages,
                                           "currentPage" => $page));
        
        $c->setPagination($page, $perPage);
           
        $this->set("webmastersCount", $totalCount);
          
        $c->addLeftJoin("sites", "sites.webmasterId", "users.userId");
        $c->addGroup("userId");
        $c->addOrder("sitesCount DESC");
        
        $webmasters = $this->user->findAll($c, "userId, email, COUNT(siteId) as sitesCount");
        $this->set("webmasters", $webmasters);
    }
    
    function banIpAction()
    {
        $this->set("bannedIps", $this->bannedIp->generateList(null, "remoteIP"));
    }
    
    function saveNewIpBanAction()
    {
        $bannedIps = preg_split("#\r?\n#", $this->request->bannedIps);
        
        foreach($bannedIps as $ip)
        {
            $ip = trim($ip);
            if(empty($ip))continue;
            
            $bannedIp = new BannedIpRecord();
            $bannedIp->remoteIp= $ip;
            $bannedIp->save();
        }
        
        $this->redirect($this->moduleLink("banIp"));    
    }
    
    function deleteIpBanAction()
    {
        if(!empty($this->request->unbanIps))
        {
            $c = new Criteria();
            $c->add("banId", $this->request->unbanIps, "IN");
            $this->bannedIp->del($c);
        }
        
        $this->redirect($this->moduleLink("banIp"));     
    }
    
    function banEmailAction()
    {
        $this->set("bannedEmails", $this->bannedEmail->generateList(null, "email")); 
    }
    
    function saveNewEmailBanAction()
    {
        $bannedEmails = preg_split("#\r?\n#", $this->request->bannedEmails);
        foreach($bannedEmails as $email)
        {
            $email = trim($email);
            if(empty($email))continue;
            
            $bannedEmail = new BannedEmailRecord();
            $bannedEmail->email = $email;
            $bannedEmail->save();
        }
        
        $this->redirect($this->moduleLink("banEmail"));
    }
    
    function deleteEmailBanAction()
    {              
        if(!empty($this->request->unbanEmails))
        {
            $c = new Criteria();
            $c->add("banId", $this->request->unbanEmails, "IN");
            $this->bannedEmail->del($c);
        }
        
        $this->redirect($this->moduleLink("banEmail")); 
    }
    
    function newsletterAction()
    {
    
    }
    
    function editWebmasterAction($webmasterId)
    {
        $c = new Criteria();
        $c->add("webmasterId", $webmasterId);
        $this->set("userSites", $this->site->findAll($c));
        $this->set("webmaster", $this->user->findByPk($webmasterId)); 
    }
    
    function deleteWebmasterAction($webmasterId)
    {
        $this->user->delByPk($webmasterId);
        $this->redirect($this->moduleLink("webmaster"));
    }
    
    function updateWebmasterAction($webmasterId)
    {
        if(isset($this->request->delete))
        {
            $this->deleteWebmasterAction($webmasterId);
        }
        else
        {                
            $this->user->changeUserEmail($webmasterId, $this->request->email);
            $this->redirect($this->moduleLink("webmaster"));
        }
    }
    
    function editModeratorAction($moderatorId)
    {
        $this->set("moderator", $this->user->findByPk($moderatorId));
    }
    
} 

?>