<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class NavigationTreeItem
{
    public $nodeId;
    public $parentId;
    public $childs = array();
    public $value;
}

class NavigationTree
{
    private $tree;
    
    function __construct()
    {
        $rootNode = new NavigationTreeItem();
        $rootNode->nodeId = 0;
        $rootNode->value = array("name" => "Root", "count" => 0);
        $this->tree[0] = $rootNode;
    }
    
    function addNode($nodeId, $parentId, $value)
    {        
        //echo "Add $nodeId, $parentId<br>";  
        
        if(!isset($this->tree[$parentId]))
        {
            $this->tree[$parentId] = new NavigationTreeItem();
        }
        
        if(!isset($this->tree[$nodeId]))
        {
            $newNode = new NavigationTreeItem();
        }
        else
        {
            $newNode = $this->tree[$nodeId];
        }
        
        $newNode->nodeId = $nodeId;
        $newNode->value = $value;
        $newNode->parentId = $parentId;
    
        $this->tree[$nodeId] = $newNode;

        $parentNode =& $this->tree[$parentId];
        $parentNode->childs[] = $nodeId; 
    }
    
    function render()
    {
        $resultList = array();
        $this->renderNode(0, array(), $resultList);
        return $resultList;  
    }
    
    function renderNode($nodeId, $parents, &$resultList, $level = 0)
    {
        $node = $this->tree[$nodeId];
        $value =& $node->value;

        if($level > 0)
        {
            $newItem = $value;

            $newItem['parents'] = $parents;
            $parents[] = $newItem; 
            
            $resultList[] = $newItem; 
        }
        
        foreach($node->childs as $childNodeId)
        {
            //echo "next";
            $this->renderNode($childNodeId, $parents, $resultList, $level + 1); 
        }     
    }
    
    function createSqlNode($nodeId, $level = 0, $parentsChain)
    {
        $node = $this->tree[$nodeId];
        $value =& $node->value;
        $out = "";

        $parentsChain[$node->nodeId] = $level;

        foreach($parentsChain as $parentId => $parentLevel)
        {
            $out .= "INSERT INTO categoryparents VALUES($parentId, ".$node->nodeId.", ".($level-$parentLevel).");\r\n"; 
        }

        foreach($node->childs as $childNodeId)
        {
            $out .= $this->createSqlNode($childNodeId, $level + 1, $parentsChain); 
        }
        
        return $out;     
    }

    function createSql()
    {
        $parentsChain = array();
        return $this->createSqlNode(0, 0, $parentsChain);    
    }
    
    function createConnection($nodeId, $level = 0, $parentId)
    {
        $node = $this->tree[$nodeId];
        $value =& $node->value;
        $out = array();
        
        if($nodeId)
        {
            $out[] = array("parentCategoryId" => $parentId, "categoryId" => $node->nodeId);
        } 
                       
        foreach($node->childs as $childNodeId)
        {
            $out = array_merge($out, $this->createConnection($childNodeId, $level + 1, $nodeId)); 
        }
        
        return $out;     
    }

    function getAllConnections()
    {
        return $this->createConnection(0, 0, 0);    
    }
    
    function optionList($nodeId, $optionLine, &$resultsList, $level = 0)
    {
        $node = $this->tree[$nodeId];
        
        $value =& $node->value;
        
        if($level > 1)
        {
            $optionLine .=" > ";
        } 
        
        if($level > 0)
        {
            $optionLine .= $value['name'];
            $resultsList[$node->nodeId] = $optionLine;
        }
        
        foreach($node->childs as $childNodeId)
        {
            $this->optionList($childNodeId, $optionLine, $resultsList, $level + 1); 
        }     
    }
    
    function getFullOptionList()
    {        
        $resultsList = array();
        $this->optionList(0, "", $resultsList);
        return $resultsList;    
    }
}

?>