<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class PaymentController extends Controller
{
    /**
    * Initailize controller set access privileges
    */
    function init()
    {        
        $this->acl->allow('webmaster', $this->name, "*");
        
        $this->acl->allow('guest', $this->name, "ipn");
        
        if(!Config::get("registrationOfWebmastersEnabled"))
        {
            $this->acl->allow('guest', $this->name, "selectPaymentOptions");
            $this->acl->allow('guest', $this->name, "makePayment");
            $this->acl->allow('guest', $this->name, "processPayment");
        }
        
        if(!$this->acl->isAllowed($this->session->get("role"), $this->name, $this->action))
        {
            $this->redirect(FrontController::getRewrittedUrl("/webmaster/logIn"));
        }
    }
    
    public function selectPaymentOptionsAction()
    {
        $packages = $this->package->findAll();

        $payPal = $this->paymentProcessor->findByPk("PayPal");
        $allopass = $this->paymentProcessor->findByPk("Allopass");
        
        foreach($packages as &$package)
        {
            $details = "";
            
            if($payPal->active)
            {
                $details .= " ".$package['amount']." ".$payPal->currency;
            }
            
            if($payPal->active && $allopass->active)$details .= " or ";
            
            if($allopass->active)
            {
                $details .= $package['allopassNumber']." Allopass";
            }
            
            $package['details'] = $details;
        }
        
        $this->set("packages", $packages);
        $c = new Criteria();
        $c->add("active", 1);
        $this->set("paymentProcessors", $this->paymentProcessor->generateList($c, "displayName"));
    }
    
    public function factoryPaymentProcessor($processorId)
    {
        switch($processorId)
        {
            case "PayPal":
                $paymentProcessor = new PayPalPaymentProcessor();
                break;
                
            case "Allopass":
                $paymentProcessor = new AllopassPaymentProcessor();
                break;
            
            default:
                $paymentProcessor = null;
        }
        
        return $paymentProcessor;
    }
    
    public function makePaymentAction()
    {
        if(empty($this->request->processorId))$this->redirect($this->moduleLink("selectPaymentOptions"));
        $processorId = $this->request->processorId;
        $package = $this->package->findByPk($this->request->packageId);
        $paymentProcessor = $this->factoryPaymentProcessor($this->request->processorId);
        
        if(empty($paymentProcessor) || empty($package))$this->redirect($this->moduleLink("selectPaymentOptions"));
        
        $paymentProcessor->setProduct($package);
        $this->set("package", $package);
        $this->set("paymentOptions", $paymentProcessor->getPaymentOptions());
    }

    
    public function processPaymentAction()
    {
        if(empty($this->request->processorId)
        || empty($this->request->packageId))$this->redirect($this->moduleLink("selectPaymentOptions"));
        
        $processorId = $this->request->processorId;
        $package = $this->package->findByPk($this->request->packageId);
        $paymentProcessor = $this->factoryPaymentProcessor($processorId);
        
        if(empty($paymentProcessor) || empty($package))$this->return404();
        
        $paymentProcessor->setProduct($package);
        $processPaymentSuccesfully = $paymentProcessor->processPayment();

        if($processorId == "PayPal")
        {
            $notifyUrl = Config::get("siteRootUrl").$this->moduleLink("ipn", false); 
            
            if(Config::get("registrationOfWebmastersEnabled"))
            {
                $returnUrl = Config::get("siteRootUrl").FrontController::getRewrittedUrl("/webmaster/loading", false);
                $cancelReturnUrl = Config::get("siteRootUrl").FrontController::getRewrittedUrl("/webmaster/submitWebsite/premium", false);
            }
            else
            {
                $returnUrl = Config::get("siteRootUrl").FrontController::getRewrittedUrl("/webmaster/loading", false);
                $cancelReturnUrl = Config::get("siteRootUrl").FrontController::getRewrittedUrl("/webmaster/submit1/premium", false);
            }
             
            $paymentProcessor->setNotifyUrl($notifyUrl);
            $paymentProcessor->setReturnUrl($returnUrl);
            $paymentProcessor->setCancelReturnUrl($cancelReturnUrl);
                
            $this->set("paymentOptions", $paymentProcessor->getPaymentOptions());
        }
        
        if($processorId == "Allopass")
        {
            $this->viewClass = "JsonView";
            
            if(!$processPaymentSuccesfully)
            {
                //set status to js part about that something is wrong with site data
                $this->set("status", "error");
                
                //set error message what is wrong
                $this->set("message", _t($paymentProcessor->getLastError()));
                return;
            }
     
            //set status to js part about site proccessing
            $this->set("status", "ok"); 

            //inform js to redirect
            if(Config::get("registrationOfWebmastersEnabled"))
                $redirectUrl = FrontController::getRewrittedUrl("/webmaster/submitWebsite/premium");
            else
                $redirectUrl = FrontController::getRewrittedUrl("/webmaster/submit1/premium");
                
            $this->set("redirectUrl", $redirectUrl);
            $this->set("message", _t("Allopass was validated sucessfully."));
        }
    }
    
    public function ipnAction()
    {
        $paymentProcessor = new PayPalPaymentProcessor();
        $paymentProcessor->handleIpn();
        
        $this->autoRender = false;
    }
}

?>