<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class SiteController extends Controller
{
    /**
    * Get random sites on index page
    */
    function randomListAction()
    {
        $cache = Cacher::getInstance();

        //get last time generated
        $generationDate = Config::get('selectionGenerationDate');

        $period = Config::get('selectionPeriod');
        $todayDate = date("Y-m-d");

        //if doesn't exists last refresh or is overtime
        $refreshNeeded = (!preg_match('#^[0-9]{4}-[0-9]{2}-[0-9]{2}$#', $generationDate)
                          || strtotime($generationDate . "+".$period." days") <= strtotime($todayDate));


        //if force refresh or cache donesn't exists
        if($refreshNeeded || ($data = $cache->load("indexRandomPages")) === null)
        {
            $data = $this->site->getIndexRandomList($refreshNeeded);
            $cache->save($data, "indexRandomPages", array("site", "setting"));
        }

        $this->set("randomSites", $data);
        $this->set("siteDetailsLink", $this->moduleLink("details/"));
    }

    /**
    * Display news sites
    */
    function newsAction()
    {
        //set adPage for ads
        Display::set("adPage", "news");

        $this->set("newSites", $this->site->getNewValidatedSites());
    }

    /**
    * Display sites inside this specified category
    */
    function categoryAction($categoryId, $name, $page)
    {
        $page = intval($page);

        //set adPage for ads
        Display::set("adPage", "category".$categoryId);

        //set current category for included menus, categories listings
        Display::set("currentCategoryId", $categoryId);
        Display::set("categoryRssHref", FrontController::getRewrittedUrl("/rss/category/$categoryId"));

        $category = $this->category->findByPk($categoryId);

        if(empty($category) || FrontController::changeFr($category->name) != $name)
        {
            return $this->return404();
        }

        //set cookie for future use during site submistion to select last category
        setcookie('lastVisitedCategoryId', $categoryId);
        
        //get validated sites in this category
        $sites = $this->site->getValidatedSitesInCategory($categoryId, $page);

        if($category->description)
        {
            //set META description in HEAD section  
            $this->set("metaDescription", utf8_substr(preg_replace("#\r?\n#", "", strip_tags($category->description)), 0, 100)." - "._t("page")." ".$page);
        }
        else
        {
            if(!empty($sites))
            {
                //set META description in HEAD section
                $this->set("metaDescription", utf8_substr(preg_replace("#\r?\n#", "", strip_tags($sites[0]['description'])), 0, 100)." - "._t("page")." ".$page);
            }
        }

        $this->set("category", $category);

        $this->set("categoryParentsData", $this->categoryParent->getParentsData($categoryId));
        $this->set("siteCategoryLink", $this->moduleLink("category/"));


        
        $this->set("sitesInCategory", $sites);

        //prepare pagination data
        $c = new Criteria();
        $c->add("categoryId", $categoryId);
        $c->add("status", "validated");

        $totalPages = ceil($this->site->getCount($c) / Config::get("sitesPerPageInCategory"));

        $this->set("pageNavigation", array("baseLink" => "/site/category/$categoryId/$name/",
                                           "totalPages" => $totalPages,
                                           "currentPage" => $page));
    }

    /**
    * Display sites which have specified keyword
    */
    function keywordAction($keywordId, $name, $page)
    {
        //if this action is disabled
        if(!Config::get('keywordsEnabled'))return $this->return404();

        //set adPage for ads
        Display::set("adPage", "keyword".$keywordId);

        //get keyword data
        $keyword = $this->keyword->findByPk($keywordId);
        $this->set("keyword", $keyword);

        if(empty($keyword) || FrontController::changeFr($keyword->keyword) != $name)
        {
            return $this->return404();
        }

        //get sites which containt this keyword
        $keywordSites = $this->site->getSitesThatOwnAKeyword($keywordId, $page);
        $this->set("keywordSites", $keywordSites);

        if(!empty($keywordSites))
        {
            //set META description in HEAD section
            $this->set("metaDescription", utf8_substr(preg_replace("#\r?\n#", "", $keywordSites[0]['description']), 0, 100)." - "._t("page")." ".$page);
        }

        //prepare pagination data
        $c = new Criteria();
        $c->add("status", "validated");
        $c->addInnerJoin("keywordsofsites", "keywordsofsites.siteId", "sites.siteId");
        $c->add("keywordsofsites.keywordId", $keywordId);

        $totalPages = ceil($this->site->getCount($c) / Config::get("sitesPerPageInKeywords"));

        $this->set("pageNavigation", array("baseLink" => "/site/keyword/$keywordId/$name/",
                                           "totalPages" => $totalPages,
                                           "currentPage" => $page));
    }

    /**
    * Display Top hits sites
    */
    function topHitsAction()
    {
        //if this action is disabled
        if(!Config::get("hitsEnabled"))return $this->return404();

        //set adPage for ads
        Display::set("adPage", "topHits");

        //will display hitsCount on included /site/item.tpl
        Display::set("hitsCountInListItem", true);

        $this->set("topHitSites", $this->site->getValidatedTopVisitedSites());
    }

    /**
    * Display Top notes sites
    */
    function topNotesAction()
    {
        //if this action is disabled
        if(!Config::get("notationsEnabled"))return $this->return404();

        //set adPage for ads
        Display::set("adPage", "topNotes");

        //will display notesCount on included /site/item.tpl
        Display::set("notesCountInListItem", true);

        //retrieve validated top notes sites
        $this->set("topNoteSites", $this->site->getValidatedTopElectedSites());
    }

    /**
    * Display Top rank sites
    */
    function topRankAction($pageRank = false)
    {
        //if this action is disabled
        if(!Config::get("topRankEnabled"))return $this->return404();

        //check correct pageRank range
        if($pageRank && ((int)$pageRank < 1 || (int)$pageRank > 10))
        {
            return $this->return404();
        }

        //set adPage for ads
        Display::set("adPage", "topRank");

        //will display pageRankImg on included /site/item.tpl
        Display::set("pageRankImgInListItem", true);
        $this->set("pageRank", $pageRank);

        //get validated top rank sites
        $this->set("topRankSites", $this->site->getValidatedTopRankSites($pageRank));
    }

    /**
    * Display Top referrers sites
    */
    function topReferrersAction()
    {
        //if this action is disabled
        if(!Config::get("topReferrersEnabled"))return $this->return404();

        //set adPage for ads
        Display::set("adPage", "topReferrers");

        //will display referrersCount on included /site/item.tpl
        Display::set("referrersCountInListItem", true);

        $this->otherReferrerSite->checkReset();
        //get validated topReferrers
        $this->set("topReferrerSites", $this->site->getValidatedTopReferrersSites());
    }

    /**
    * Display site details page
    */
    function detailsAction($siteId, $siteTitle = "")
    {
        //set adPage for ads
        Display::set("adPage", "site".$siteId);

        //get site data
        $site = $this->site->getSiteWithDetails($siteId);
        if(empty($site) || $site->status != "validated")
        {
            return $this->return404();
        }

        //set category id where site is placed to retrieve predefinitions in ads module
        Display::set("adSiteCategoryId", $site->categoryId);

        //set site rss href
        Display::set("siteRssHref", FrontController::getRewrittedUrl("/rss/site/$siteId"));

        //check passed siteTitle is url is correct
        if(FrontController::changeFr($site->siteTitle) != $siteTitle)
        {
            $this->return404();
        }

        //get category path to this site
        $site->categoryParentsData = $this->categoryParent->getParentsData($site->categoryId);

        //get site keywords
        $site->keywords = $site->getKeywords();

        //set META keywords in HEAD section
        $this->set("metaKeywords", implode(", ", $site->keywords));

        //set META description in HEAD section
        $this->set("metaDescription", utf8_substr(preg_replace("#\r?\n#", "", $site->description), 0, 200));

        //get google details
        $googleStats = new GoogleStats();
        $results = $googleStats->getGoogleDetailsOfSiteIfCached($site->url);

        //if results exists pass them to template
        if($results)
        {
            $site->pageRank = $results['pageRank'];
            $site->backlinksCount = $results['backlinksCount'];
            $site->indexedPagesCount = $results['indexedPagesCount'];
        }
        else
        {
            //if no use ajax to reduce site generation time
            $this->set("ajaxGoogleDetails", true);
        }

        if(Config::get("showRandomSitesInDetails"))
        {
            $this->set("randomSites", $site->getSimilarSites());
        }

        //set site data
        $this->set("site", $site);
        
        if(Config::get("googleMapEnabled"))
        {
            $this->set("googleMap", $site->getGoogleMap());
        }
        
        if(Config::get("remoteRssParsingEnabled") && !empty($site['rssFeedOfSite']))
        {
            define("MAGPIE_OUTPUT_ENCODING", "UTF-8");
            define("MAGPIE_DETECT_ENCODING", true);

            // Define cache's maximum age
            define('MAGPIE_CACHE_AGE', 60*60*24 * intval(Config::get('magpieRssCacheMaxAgeDays')));

            require_once(CODE_ROOT_DIR.'components/magpierss/rss_fetch.php');
            $rss = @fetch_rss($site['rssFeedOfSite']);

            if(!empty($rss))
            {
                $items = array_slice($rss->items, 0, Config::get("numberOfItemsForRssParsing"));

                // Strip tags from each item's description
                array_walk($items, create_function('&$v,$k', 'if (isset($v[\'description\'])) { $v[\'description\'] = strip_tags(html_entity_decode($v[\'description\'], ENT_COMPAT, \'UTF-8\')); }'));

                $remoteRss = array("items" => $items, "channel" => $rss->channel);

                $this->set("remoteRss", $remoteRss);
            }
        }

    }

    /**
    * Retrieve google details and return result to ajaax query
    */
    function getGoogleDetailsAction()
    {
        $site = $this->site->findByPk($this->request->siteId, "url");
        if(empty($site))return $this->return404();
        
        $googleStats = new GoogleStats();
        $results = $googleStats->getGoogleDetailsOfSite($site->url);

        $this->set($results);
        $this->viewClass = "JsonView";
    }

    /**
    * Display site problemPopup
    */
    function problemPopupAction($siteId)
    {
        $site = $this->site->findByPk($siteId, "siteId");
        if(empty($site))return $this->return404();

        $displayWarning = false;
        $displayForm = false;
        $cookieName = md5('siteProblem' . $siteId);

        if(isset($_COOKIE[$cookieName]) && $_COOKIE[$cookieName] == date('d.m.Y'))
        {
            $displayForm = false;
            $displayWarning = true;
        }
        else
        {
            //if post value
            if(isset($this->request->problemValue))
            {
                //dont display form
                $displayForm = false;

                //if results was Yes save it to db
                if($this->request->problemValue)
                {
                    setcookie(
                                $cookieName,
                                date('d.m.Y'),
                                mktime ( 0, 0, 0, date('n'), date('j'), date('Y')) + 60 * 60 * 24);

                    $site->problemExists = 1;
                    $site->save();
                }
            }
            else
            {
                //display form to select have problem
                $this->set("siteId", $siteId);
                $displayForm = true;
            }
        }

        $this->set("displayForm", $displayForm);
        $this->set("displayWarning", $displayWarning);
    }
    
    /**
    * Display sites which contain tag
    */
    function tagAction($tagId, $tag, $page = 1)
    {
        $searchTag = $this->searchTag->findByPk($tagId);

        if(empty($searchTag) || FrontController::changeFr($searchTag->tag) != $tag || $searchTag->banned)
        {
            return $this->return404();
        }
        
        //set adPage for ads
        Display::set("adPage", "search");
        $searchedSites = array();

        //get sites which containt searched phrase
        $searchedSites = $this->site->searchValidated($tag, $page);
        
        if(!empty($searchedSites))
        {
            //set META description in HEAD section
            $this->set("metaDescription", utf8_substr(preg_replace("#\r?\n#", "", $searchedSites[0]['description']), 0, 100)." - "._t("page")." ".$page);
        }

        //prepare data for pagination
        $totalPages = ceil($this->site->getSearchValidatedCount($tag) / Config::get("sitesPerPageInSearch"));
    
        //set founded sites and paginations data
        $this->set("searchTag", $searchTag);
        $this->set("searchedSites", $searchedSites);
        $this->set("pageNavigation", array("baseLink" => "/site/tag/$tagId/".FrontController::changeFr($tag)."/",
                                           "totalPages" => $totalPages,
                                           "currentPage" => $page));

    }

    /**
    * Display sites which containt searched phrase
    */
    function searchAction($page = 1)
    {
        //set adPage for ads
        Display::set("adPage", "search");
        $searchedSites = array();

        //if new search
        if(isset($this->request->phrase))
        {
            //save search phrase for pagination work
            $this->session->set("phrase", $this->request->phrase);
        }

        //get last searched phrase
        $phrase = $this->session->get("phrase");

        //if something was searched
        if($phrase)
        {
            //set phrase to highlight in site/item.tpl
            Display::set("highlightKeywords", preg_split("#\s+#", $phrase));

            //get sites which containt searched phrase
            $searchedSites = $this->site->searchValidated($phrase, $page);
            
            //if this phrase have matched sites save tag
            if(!empty($searchedSites))
            {
                $this->searchTag->addSearchTag($phrase, false);
            }

            //prepare data for pagination
            $totalPages = ceil($this->site->getSearchValidatedCount($phrase) / Config::get("sitesPerPageInSearch"));
        }
        else
        {
            //nothing was searched
            $searchedSites = array();
            $totalPages = 0;
            $page = 1;
        }

        //set founded sites and paginations data
        $this->set("searchedSites", $searchedSites);
        $this->set("pageNavigation", array("baseLink" => "/site/search/",
                                           "totalPages" => $totalPages,
                                           "currentPage" => $page));

    }

    /**
    * Action executed when user click on image, site link before he is redirected to it
    */
    function visitAction()
    {
        $this->autoRender = false;
        if(empty($this->request->siteId))return;

        $siteId = $this->request->siteId;
        $currentDate = date('d.m.Y');
        $cookieName = md5('site' . $siteId);

        //if can count this vote
        $ip = $this->request->getIp();
        $type = "vis";

        if((!isset($_COOKIE[$cookieName]) || $_COOKIE[$cookieName] != $currentDate)
        && !$this->visit->exists($ip, $type, $siteId, 24))
        {
            $visit = new VisitRecord();
            $visit->ip = $ip;
            $visit->type = $type;
            $visit->id = $siteId;
            $visit->save();

            //set cookie to next midnight
            setcookie(
                $cookieName,
                $currentDate,
                mktime ( 0, 0, 0, date('n'), date('j'), date('Y')) + 60 * 60 * 24);

            //save vote
            $this->hit->insertHit($ip, $siteId);
        }

    }

    /**
    * Action executed when donesn't exists cached site image and caching is ON
    * Retrieve image from external source and store it
    */
    function getThumbAction($siteId)
    {
        $site = $this->site->findByPk($siteId, "siteId, url, imageSrc");

        if(empty($site))
        {
            $this->return404();
        }

        //download image and sotre it to cache
        if($site->downloadAndCacheThumb())
        {
            //get web path to that
            $site->updateImageSrc(true);
            $imageLocation = $site->imageSrc;
        }
        else
        {
            //if something goes wrong, generator timouts, wrong image display default
            $imageLocation = $site->getDefaultImageSrc();
        }

        //redirect to image location
        $this->redirect($imageLocation);
    }

}
?>