<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class SearchTagModel extends Model
{   
    protected $primaryKey = "tagId";
    
    public function handleSearchEngineReferrer($url)
    {
        if(!preg_match("#http://www.google.com/search\?.*q=([^&]+)#", $url, $match))return;
        
        $tag = $match[1];
        $this->addSearchTag($tag);
    }
    
    public function addSearchTag($tag, $checkSearchResult = true)
    {
        $c = new Criteria();
        $c->add("tag", $tag);
        $searchTag = $this->find($c);
        $updateSearchTimes = true;
        
        if(!$searchTag)
        {
            if($checkSearchResult && !$this->site->getSearchValidatedCount($tag))return;
            
            $searchTag = new SearchTagRecord();
            $searchTag->tag = $tag;
            $searchTag->searchTimes = 0;
        }
        else
        {
            $ip = Request::getInstance()->getIp();
            $type = "tag";
            
            if(!$this->visit->exists($ip, $type, $searchTag->tagId, 24))
            {
                $visit = new VisitRecord();
                $visit->ip = $ip;
                $visit->type = $type;
                $visit->id = $searchTag->tagId;
                $visit->save();
            }
            else
            {
                $updateSearchTimes = false;
            }
        }
        
        if($updateSearchTimes)
        {
            $searchTag->searchTimes = $searchTag->searchTimes + 1;
            $searchTag->save();
        }
    }
    
    private function sortTag($a, $b)
    {
        return strcmp($a['tag'], $b['tag']);
    }
    
    public function getTagCloud()
    {
        $c = new Criteria();
        $c->addOrder("searchTimes DESC");
        $c->add("banned", "0");
        $c->setLimit(Config::get("numberOfTagInTagCloud"));
        $tags = $this->findAll($c);
        
        if(count($tags) < min(Config::get("numberOfTagInTagCloud"), Config::get("minNumberOfTagInTagCloud")))return array();
        
        usort($tags, array($this, "sortTag"));

        $maxSearchTimes = 0;
        $minSearchTimes = pow(2, 32);
        
        $minSize = 1;
        $maxSize = 10;

        foreach($tags as $tag)$maxSearchTimes = max($maxSearchTimes, $tag['searchTimes']);
        foreach($tags as $tag)$minSearchTimes = min($minSearchTimes, $tag['searchTimes']);
        
        $range = $maxSearchTimes - $minSearchTimes;
        if($range == 0)$range = 1;
        $step = ($maxSize - $minSize) / $range;
        
        foreach($tags as &$tag)
        {
            $tag['size'] = intval($minSize + (($tag['searchTimes'] - $minSearchTimes) * $step));  
        }
        
        return $tags;
    }
    
}

class SearchTagRecord extends ModelRecord
{
    public function save()
    {
        parent::save();
        
        $cache = Cacher::getInstance();
        $cache->clean("tag", array("searchTag"));
    }
}

?>